/*	low level i/o routines for ah format records
 *		-- witte	6 june 85
 */

#include <stdio.h>
#include <sys/ioctl.h>
#include <rpc/rpc.h>
#include <math.h>

#include "ahhead.h"

/* ah error processing */

int	ah_errno = 0;
int	ah_nerr = 10;

/* ah error numbers */
#define	AE_RHED		1	/* error reading header	*/
#define	AE_DTYPE	2	/* bad data type	*/
#define	AE_WHED		3	/* error writing header	*/
#define	AE_RDATA	4	/* error reading data	*/
#define	AE_WDATA	5	/* error writing data	*/
#define	AE_WRECORD	6	/* error writing record	*/
#define	AE_RRECORD	7	/* error reading record	*/
#define	AE_TTYOUT	8	/* binary going to tty	*/
#define	AE_TTYIN	9	/* binary coming from tty	*/

/* ah errlist */

char	*ah_errlist[] = {
		"no error",		/* 0	no error	*/
		"read header error",	/* 1	AE_RHED		*/
		"bad data type",	/* 2	AE_DTYPE	*/
		"write header error",	/* 3	AE_WHED		*/
		"read data error",	/* 4	AE_RDATA	*/
		"write data error",	/* 5	AE_WDATA	*/
		"write record error",	/* 6	AE_WRECORD	*/
		"read record error",	/* 7	AE_RRECORD	*/
		"tty can't get binary",	/* 8	AE_TTYOUT	*/
		"tty can't send binary"	/* 9	AE_TTYIN	*/
			};

/*	xdr_gethead
 *		gets the next header from the xdr stream pointed to by
 *		xdrs and returns this header in the structure head.
 *		xdrs is assumed to be positioned at the next header,
 *		and does not search.
 *	returns:
 *			1		->	no error
 *			-1		->	not enough head to read
 *			-2		->	bad data type
 */
int	xdr_gethead(head,xdrs)
	XDR	*xdrs;
	ahhed	*head;
{
	int	ierr = 0;

	if((ierr = xdr_ahhead(xdrs, head)) == 1)
	{
		if((head->record.type < TYPEMIN) || (head->record.type > TYPEMAX))
		{
			get_null_head(head);
			ierr = -2;	/* bad data type */
			ah_errno= AE_DTYPE;
		}
	}
	else		/* not enough head */
	{
		get_null_head(head);
		ierr = -1;
		ah_errno= AE_RHED;
	}
	return(ierr);
}



/*	xdr_puthead
 *		writes the header head onto the xdr stream pointed to by
 *		xdrs.
 *	returns:
 *			1		->	no error
 *			-1		->	error writing header
 */
int	xdr_puthead(head,xdrs)
	XDR	*xdrs;
	ahhed	*head;
{
	int	ierr = 0;

	if((ierr= xdr_ahhead(xdrs, head)) != 1)
	{
		ah_errno= AE_WHED;
		ierr= -1;
	}
	return(ierr);
}



/*	xdr_tohead
 *		positions the read/write head to the beginning of the
 *		n-th header in the xdr stream pointed to by xdrs.
 *	returns:
 *			n	->	no error
 *			-1	->	not enough heads
 *			-2	->	bad seek
 */
int	xdr_tohead(n,xdrs)
	XDR	*xdrs;
	int	n;
{
	ahhed	head;
	int	i,ierr,j;
	float	float_dum;
	double	double_dum;
	complex complex_dum;
	tensor	tensor_dum;

/* be warned: the following xdr_setpos call may not work at all 	*/
/* depending on the stream.  The use of 0 to get to the beginning 	*/
/* works empirically, but is not documented  ... sigh	- dws		*/
	xdr_setpos(xdrs, (u_int) 0);

	for(i=1; i<n; ++i)
	{
		if(xdr_gethead(&head,xdrs) == 1)
		{
			switch (head.record.type) {
			case FLOAT:
				for (j = 0; j < head.record.ndata; j++) {
					if (!xdr_float(xdrs, &float_dum)) {
						ierr = -2;	/* bad seek */
						ah_errno= AE_RHED;
						return(ierr);
					}
				}

				break;
			case COMPLEX:
			case VECTOR:
				for (j = 0; j < head.record.ndata; j++) {
					if (!xdr_float(xdrs, &complex_dum.i) ||
					    !xdr_float(xdrs, &complex_dum.r)) {
						ierr = -2;	/* bad seek */
						ah_errno= AE_RHED;
						return(ierr);
					}
				}

				break;
			case TENSOR:
				for (j = 0; j < head.record.ndata; j++) {
					if (!xdr_float(xdrs, &tensor_dum.xx) ||
					    !xdr_float(xdrs, &tensor_dum.yy) ||
					    !xdr_float(xdrs, &tensor_dum.xy)) {
						ierr = -2;	/* bad seek */
						ah_errno= AE_RHED;
						return(ierr);
					}
				}
				break;
			case 5:
				for (j = 0; j < 4 * head.record.ndata; j++) {
					if (!xdr_float(xdrs, &float_dum)) {
						ierr = -2;	/* bad seek */
						ah_errno= AE_RHED;
						return(ierr);
					}
				}
				break;
			case DOUBLE:
				for (j = 0; j < head.record.ndata; j++) {
					if (!xdr_double(xdrs, &double_dum)) {
						ierr = -2;	/* bad seek */
						ah_errno= AE_RHED;
						return(ierr);
					}
				}
				break;
			default:
				ierr = -2;	/* bad seek */
				ah_errno= AE_DTYPE;
				return(ierr);
			}
		}
		else
		{
			ierr = -1;	/* not enough head */
			ah_errno= AE_RHED;
			return(ierr);
		}
	}
	return(i);	/* success */
}



/*	xdr_getdata
 *		reads from the xdr stream pointed to by xdrs into
 *		the array pointed to by array.  It assumes that
 *		the read/write head is positioned correctly 
 *		(i.e., right after the header), and does not
 *		search.  Works for any allowed data type.
 *	returns:
 *			number of elements read	->	OK
 *			-1			->	error
 */
int	xdr_getdata(head,array,xdrs,test)
	ahhed	*head;
	char	*array;
	XDR	*xdrs;
	float *test;

{
	int ierr = 0;
	float *pfloat;
	double	*pdouble;
	complex *pcomplex;
	tensor	*ptensor;
	int i;

	switch(head->record.type) {
	case FLOAT:
		pfloat = (float *) array;
		for (i = 0; i < head->record.ndata; i++) {
			if (! xdr_float(xdrs, pfloat++) ) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;
	case COMPLEX:
	case VECTOR:
		pcomplex = (complex *) array;
		for (i = 0; i < head->record.ndata; i++) {
			if (!xdr_float(xdrs, &(pcomplex->r)) ||
			    !xdr_float(xdrs, &(pcomplex++->i))) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;
	case TENSOR:
		ptensor = (tensor *) array;
		for (i = 0; i < head->record.ndata; i++) {
			if (!xdr_float(xdrs, &(ptensor->xx)) ||
			    !xdr_float(xdrs, &(ptensor->yy)) ||
			    !xdr_float(xdrs, &(ptensor++->xy))) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;
	case 5:
		pfloat = (float *) array;
		for (i = 0; i < 4 * head->record.ndata; i++) {
			if (! xdr_float(xdrs, pfloat++) ) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;
	case DOUBLE:
		pdouble = (double *) array;
		for (i = 0; i < head->record.ndata; i++) {
			if (! xdr_double(xdrs, pdouble++) ) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;
	default:
		ierr = -1;
		ah_errno= AE_DTYPE;
		return(ierr);
	}
	return(ierr);
}


/*	xdr_putdata
 *		writes array to the xdr stream pointed to by xdrs.
 *		Works for any allowed data type.
 *	returns:
 *			number of elements written	->	OK
 *			-1			->	error
 */
int	xdr_putdata(head,array,xdrs)
	ahhed	*head;
	char	*array;
	XDR	*xdrs;
{
	int	ierr = 0;
	float *pfloat;
	double	*pdouble;
	complex *pcomplex;
	tensor	*ptensor;
	int i;

	switch(head->record.type) {
	case FLOAT:
		pfloat = (float *) array;
		for (i = 0; i < head->record.ndata; i++) {
			if (! xdr_float(xdrs, pfloat++) ) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;
	case COMPLEX:
	case VECTOR:
		pcomplex = (complex *) array;
		for (i = 0; i < head->record.ndata; i++) {
			if (!xdr_float(xdrs, &(pcomplex->r)) ||
			    !xdr_float(xdrs, &(pcomplex++->i))) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;
	case TENSOR:
		ptensor = (tensor *) array;
		for (i = 0; i < head->record.ndata; i++) {
			if (!xdr_float(xdrs, &(ptensor->xx)) ||
			    !xdr_float(xdrs, &(ptensor->yy)) ||
			    !xdr_float(xdrs, &(ptensor++->xy))) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;
	case 5:
		pfloat = (float *) array;
		for (i = 0; i < 4 * head->record.ndata; i++) {
			if (! xdr_float(xdrs, pfloat++) ) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;
	case DOUBLE:
		pdouble = (double *) array;
		for (i = 0; i < head->record.ndata; i++) {
			if (! xdr_double(xdrs, pdouble++) ) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;

	default:
		ierr = -1;
		ah_errno= AE_DTYPE;
		return(ierr);
	}
	return(ierr);
}


/*	xdr_putrecord
 *		writes head and array to the xdr stream pointed to by xdrs.
 *		Works for any allowed data type.
 *	returns:
 *			0	->	OK
 *			-1	->	error writing header
 *			-2	->	error writing data
 */
int	xdr_putrecord(head,array,xdrs)
	ahhed	*head;
	char	*array;
	XDR	*xdrs;
{
	int	ierr = 0;

	(xdr_puthead(head,xdrs) == 1) ? ((xdr_putdata(head,array,xdrs) < 0) ? (ierr = -2) : (ierr = 0)) : (ierr = -1);
	if(ierr)
		ah_errno= AE_WRECORD;

	return(ierr);
}


/*	xdr_getrecord
 *		gets header and data from the xdr stream pointed to by
 *		xdrs and puts them in head and array.  It assumes
 *		that the read/write head is positioned at the beginning
 *		of the header, and does not search.  Obviously, calling
 *		routine must have allocated enough space.
 *	returns:
 *			0	->	OK
 *			-1	->	error reading header
 *			-2	->	error reading data
 */
int	xdr_getrecord(head,array,xdrs)
	ahhed	*head;
	char	*array;
	XDR	*xdrs;
{
	int	ierr = 0;

	(xdr_gethead(head,xdrs) == 1) ? ((xdr_getdata(head,array,xdrs) < 0) ? (ierr = -2) : (ierr = 0)) : (ierr = -1);
	if(ierr)
		ah_errno= AE_RRECORD;
	return(ierr);
}

/*
 *	xdr_getrecord2
 *		gets header and data from the xdr stream pointed to by
 *		xdrs and puts them in head and array.  It assumes
 *		that the read/write head is positioned at the beginning
 *		of the header, and does not search (although it does
 *		some error checking).  Space for array is allocated, so
 *		be sure to pass a pointer to the data pointer. Got it?
 *	returns:
 *			0	->	ok
 *			-1	->	error reading record
 *			-2	->	error allocating space for data
 */
int	xdr_getrecord2(head,array,xdrs)
	ahhed	*head;
	char	**array;
	XDR	*xdrs;
{
	int	ierr = 0;
	int	xdr_gethead();
	char	*mkdatspace();

	if(xdr_gethead(head, xdrs) != 1) {
		ierr = -1;
		return(ierr);
	}

	*array= mkdatspace(head);
	if(*array == NULL)
	{
		ierr= -2;
		return(ierr);
	}

	if(xdr_getdata(head,*array,xdrs) < 0)
		ierr= -1;

	return(ierr);
}


/*	xdr_gogethead
 *		gets n-th header from the xdr stream pointed to by
 *		xdrs and returns this header in the structure
 *		head.
 *	returns:
 *			0	->	OK
 *			-1	->	stream not long enough
 *			-2	->	error reading header
 */
int	xdr_gogethead(n,head,xdrs)
	int	n;
	ahhed	*head;
	XDR	*xdrs;
{
	int	ierr = 0;

	(xdr_tohead(n,xdrs) == n) ? ((xdr_gethead(head,xdrs) < 1) ? (ierr = -2) : (ierr = 0)) : (ierr = -1);
	return(ierr);
}


/*	xdr_gogetrecord
 *		gets n-th record (header and data) from the xdr stream
 *		pointed to by xdrs and places it in head and array.
 *		Calling routine must allocate enough space.
 *	returns:
 *			0	->	OK
 *			-1	->	stream not long enough
 *			-2	->	error reading record
 */
int	xdr_gogetrecord(n,head,array,xdrs)
	int	n;
	ahhed	*head;
	char	*array;
	XDR	*xdrs;

{
	int	ierr = 0;

	(xdr_tohead(n,xdrs) == n) ? ((xdr_getrecord(head,array,xdrs) < 0) ? (ierr = -2) : (ierr = 0)) : (ierr = -1);
	return(ierr);
}






xdr_ahhead(xdrsp, ahheadp)
XDR *xdrsp;
ahhed *ahheadp;
{
	u_int l;
	char **pp, *p;
	float **ppf, *pf;

	l = CODESIZE;
	p = ahheadp->station.code;
	pp = &p;
	if (!xdr_bytes(xdrsp, pp, &l, (u_int) CODESIZE))
		return(0);
	l = CHANSIZE;
	p = ahheadp->station.chan;
	pp = &p;
	if (!xdr_bytes(xdrsp, pp, &l, CHANSIZE))
		return(0);
	l = STYPESIZE;
	p = ahheadp->station.stype;
	pp = &p;
	if (!xdr_bytes(xdrsp, pp, &l, STYPESIZE))
		return(0);
	if (!xdr_float(xdrsp, &ahheadp->station.slat))
		return(0);
	if (!xdr_float(xdrsp, &ahheadp->station.slon))
		return(0);
	if (!xdr_float(xdrsp, &ahheadp->station.elev))
		return(0);
	if (!xdr_float(xdrsp, &ahheadp->station.DS))
		return(0);
	if (!xdr_float(xdrsp, &ahheadp->station.A0))
		return(0);
	for (l = 0; l < NOCALPTS; l++) {
		if (!xdr_float(xdrsp, &ahheadp->station.cal[l].pole.r))
			return(0);
		if (!xdr_float(xdrsp, &ahheadp->station.cal[l].pole.i))
			return(0);
		if (!xdr_float(xdrsp, &ahheadp->station.cal[l].zero.r))
			return(0);
		if (!xdr_float(xdrsp, &ahheadp->station.cal[l].zero.i))
			return(0);
	}
	if (!xdr_float(xdrsp, &ahheadp->event.lat))
		return(0);
	if (!xdr_float(xdrsp, &ahheadp->event.lon))
		return(0);
	if (!xdr_float(xdrsp, &ahheadp->event.dep))
		return(0);
	if (!xdr_short(xdrsp, &ahheadp->event.ot.yr))
		return(0);
	if (!xdr_short(xdrsp, &ahheadp->event.ot.mo))
		return(0);
	if (!xdr_short(xdrsp, &ahheadp->event.ot.day))
		return(0);
	if (!xdr_short(xdrsp, &ahheadp->event.ot.hr))
		return(0);
	if (!xdr_short(xdrsp, &ahheadp->event.ot.mn))
		return(0);
	if (!xdr_float(xdrsp, &ahheadp->event.ot.sec))
		return(0);
	l = COMSIZE;
	p = ahheadp->event.ecomment;
	pp = &p;
	if (!xdr_bytes(xdrsp, pp, &l, COMSIZE))
		return(0);
	if (!xdr_short(xdrsp, &ahheadp->record.type))
		return(0);
	if (!xdr_long(xdrsp, &ahheadp->record.ndata))
		return(0);
	if (!xdr_float(xdrsp, &ahheadp->record.delta))
		return(0);
	if (!xdr_float(xdrsp, &ahheadp->record.maxamp))
		return(0);
	if (!xdr_short(xdrsp, &ahheadp->record.abstime.yr))
		return(0);
	if (!xdr_short(xdrsp, &ahheadp->record.abstime.mo))
		return(0);
	if (!xdr_short(xdrsp, &ahheadp->record.abstime.day))
		return(0);
	if (!xdr_short(xdrsp, &ahheadp->record.abstime.hr))
		return(0);
	if (!xdr_short(xdrsp, &ahheadp->record.abstime.mn))
		return(0);
	if (!xdr_float(xdrsp, &ahheadp->record.abstime.sec))
		return(0);
	if (!xdr_float(xdrsp, &ahheadp->record.rmin))
		return(0);
	l = COMSIZE;
	p = ahheadp->record.rcomment;
	pp = &p;
	if (!xdr_bytes(xdrsp, pp, &l, COMSIZE))
		return(0);
	l = LOGSIZE;
	p = ahheadp->record.log;
	pp = &p;
	if (!xdr_bytes(xdrsp, pp, &l, LOGSIZE))
		return(0);
	l = NEXTRAS;
	p = (char *) ahheadp->extra;
	pp = &p;
	if (!xdr_array(xdrsp, pp, &l, NEXTRAS, sizeof(float),
		xdr_float))
		return(0);
	

	return(1);
}

		/*  fortran interface routine */

static	XDR xdr_file[250];
static  FILE *file[250];
static int flag_open[250], flag_mode[250];

int xdr_ahopen(iunit,file_name,mode)
	char *file_name;
	int iunit,mode;
{
	FILE *fopen();
	int ierr=0;
	if(iunit > 250) {
		ierr = -1;
		fprintf(stderr,"unit %d is to big!\n",iunit);
		return(ierr);
	}
	if(flag_open[iunit] == 1) {
		ierr = -2;
		fprintf(stderr,"unit %d is open and cannot be open again!\n",iunit);
		return(ierr);
	}
	if(mode > 1){
		ierr = -4;
		fprintf(stderr,"unit %d cannot be opened in r/w mode for xdr!\n",iunit);
		return(ierr);
	}
	fprintf(stderr,"iunit %d \t mode %d \n",iunit,mode);
	if(mode == 0) {
		/* Open file_in */
		if((file[iunit]=fopen(file_name,"r")) == NULL) {
			fprintf(stderr,"error in opening %s --- in r mode\n",file_name);
			ierr = -3;
			return(ierr);
		}	
		/* Open XDR streams for stdio */
		flag_open[iunit] = 1;
		flag_mode[iunit] = 0;
		xdrstdio_create(&xdr_file[iunit], file[iunit], XDR_DECODE);
		fprintf(stderr,"opening %s in r+ mode\n",file_name);
	}
	if(mode == 1) {	
		/* Open file_out */
		if((file[iunit]=fopen(file_name,"w+")) == NULL){
			fprintf(stderr,"error in opening %s in w+ mode\n",file_name);
			ierr = -3;
			return(ierr);
		}	
		/* Open XDR streams for stdout */
		flag_open[iunit] = 1;
		flag_mode[iunit] = 1;
		xdrstdio_create(&xdr_file[iunit], file[iunit], XDR_ENCODE);
		fprintf(stderr,"opening %s in w+ mode\n",file_name);
	}
	ierr=0;
	return(ierr);
}
int xdr_ahclose(iunit)
	int iunit;
{
	int ierr = 0;
	if(iunit > 250) {
		ierr=3;
		fprintf(stderr,"unit %d is to big!\n",iunit);
		return(ierr);
	}
	if(flag_open[iunit] != 1) {
		ierr=5;
		fprintf(stderr,"unit %d is not open and cannot be closed!\n",iunit);
		return(ierr);
	}
	xdr_destroy(&xdr_file[iunit]);
	flag_open[iunit] = 0;
	return(ierr);
}
int xdr_rewind(iunit,n)
int iunit,n;
{
	int ierr = 0;
	/* be warned: the following xdr may not work at all depending on the stream
	. The use of 0 to get to the begining works empirically but is not documented!!! */
	ierr=xdr_tohead(n,&xdr_file[iunit]);
	return(ierr);
}
int xdr_rdahhead(iunit,
	codeah,chanah,typeah,slatah,slonah,elevah,dsah,a0ah,xnpah,nu1,xnzah,nu2,
	pz,elatah,elonah,edepah,ieyrah,iemoah,iedaah,iehrah,iemnah,esecah,ecomah,itypah,ndatah,
	deltah,amaxah,isyrah,ismoah,isdaah,ishrah,ismnah,ssecah,rminah,scomah,logah,dumah)

	char            codeah[CODESIZE]; /* station code         */
	char            chanah[CHANSIZE]; /* lpz,spn, etc.        */
	char            typeah[STYPESIZE];/* wwssn,hglp,etc.     */
	float           *slatah;           /* station latitude     */
	float           *slonah;           /*    "    longitude    */
	float           *elevah;           /*    "    elevation    */
	float           *dsah;    /* gain */
	float           *a0ah;    /* normalization */
	float 		*xnpah,*xnzah; /* number of pole & zero */
	int 		*nu1,*nu2;     /* filler */
	complex 	pz[2*(NOCALPTS-1)];      /* calibration info */
	float           *elatah;            /* event latitude       */
	float           *elonah;            /*   "   longitude      */
	float           *edepah;            /*   "   depth          */
	short           *ieyrah;     /* year         */
	short           *iemoah;     /* month        */
	short           *iedaah;    /* day          */
	short           *iehrah;     /* hour         */
	short           *iemnah;     /* minute       */
	float           *esecah;    /* second       */
	char            ecomah[COMSIZE];      /*      comment line */
	short           *itypah;   /* data type (int,float,...)    */
	long            *ndatah;  /* number of samples            */
	float           *deltah;  /* sampling interval            */
	float           *amaxah; /* maximum amplitude of record  */
	short           *isyrah;     /* year         */
	short           *ismoah;     /* month        */
	short           *isdaah;    /* day          */
	short           *ishrah;     /* hour         */
	short           *ismnah;     /* minute       */
	float           *ssecah;    /* second       */
	float           *rminah;   /* minimum value of abscissa    */
	char            scomah[COMSIZE];      /* comment line      */
	char            logah[LOGSIZE]; /* log of data manipulations */
	float           dumah[NEXTRAS]; /* freebies */

	int iunit;
{
	ahhed head;
	int i,j,ierr=0;

	if(iunit > 250) {
		ierr = -2;
		fprintf(stderr,"unit %d is to big!\n",iunit);
		return(ierr);
	}
	if(flag_open[iunit] != 1) {
		ierr = -3;
		fprintf(stderr,"unit %d is not open!\n",iunit);
		return(ierr);
	}
	if(flag_mode[iunit] != 0) {
		ierr = -4;
		fprintf(stderr,"read is not possible on unit %d!\n",iunit);
		return(ierr);
	}
	if(xdr_gethead(&head,&xdr_file[iunit]) == 1)
	{
		ierr=0;
		head.station.code[5]= '\0';
		head.station.chan[5]= '\0';
		head.station.stype[7]= '\0';
		head.event.ecomment[79]= '\0';
		head.record.rcomment[79]= '\0';
		head.record.log[LOGSIZE-1]= '\0';

		strcpy(codeah,head.station.code);
		strcpy(chanah,head.station.chan);
		strcpy(typeah,head.station.stype);
		*slatah=head.station.slat;
		*slonah=head.station.slon;
		*elevah=head.station.elev;
		*dsah=head.station.DS;
		*a0ah=head.station.A0;
		*xnpah = head.station.cal[0].pole.r;
		*nu1=(int)head.station.cal[0].pole.i;
		*xnzah = head.station.cal[0].zero.r;
		*nu2=(int)head.station.cal[0].zero.i;
		j=1;
		for (i=0;i<2*(NOCALPTS-1);i+=2){
			pz[i]=head.station.cal[j].pole;
			pz[i+1]=head.station.cal[j].zero;
			j++;
		}
		*elatah=head.event.lat;
		*elonah=head.event.lon;
		*edepah=head.event.dep;
		*ieyrah=head.event.ot.yr; 
		*iemoah=head.event.ot.mo;
		*iedaah=head.event.ot.day;
		*iehrah=head.event.ot.hr;
		*iemnah=head.event.ot.mn;
		*esecah=head.event.ot.sec;
		*itypah=head.record.type;
		*ndatah=head.record.ndata;
		*deltah=head.record.delta;
		*amaxah=head.record.maxamp;
		*isyrah=head.record.abstime.yr; 
		*ismoah=head.record.abstime.mo;
		*isdaah=head.record.abstime.day;
		*ishrah=head.record.abstime.hr;
		*ismnah=head.record.abstime.mn;
		*ssecah=head.record.abstime.sec;
		*rminah=head.record.rmin;
	     	strcpy(scomah,head.record.rcomment);
		strcpy(logah,head.record.log);  
		for(i=0;i<NEXTRAS-1;i++){
			dumah[i]=head.extra[i];
		} 
		dumah[NEXTRAS]=head.extra[NEXTRAS];
		ierr=1024;
	} 
	return(ierr);
}
int xdr_rdahrecord(test,iunit,
	codeah,chanah,typeah,slatah,slonah,elevah,dsah,a0ah,xnpah,nu1,xnzah,nu2,
	pz,elatah,elonah,edepah,ieyrah,iemoah,iedaah,iehrah,iemnah,esecah,ecomah,itypah,ndatah,
	deltah,amaxah,isyrah,ismoah,isdaah,ishrah,ismnah,ssecah,rminah,scomah,logah,dumah,array)

	char            codeah[CODESIZE]; /* station code         */
	char            chanah[CHANSIZE]; /* lpz,spn, etc.        */
	char            typeah[STYPESIZE];/* wwssn,hglp,etc.     */
	float           *slatah;           /* station latitude     */
	float           *slonah;           /*    "    longitude    */
	float           *elevah;           /*    "    elevation    */
	float           *dsah;    /* gain */
	float           *a0ah;    /* normalization */
	float 		*xnpah,*xnzah; /* number of pole & zero */
	int 		*nu1,*nu2;     /* filler */
	complex 	pz[2*(NOCALPTS-1)];      /* calibration info */
	float           *elatah;            /* event latitude       */
	float           *elonah;            /*   "   longitude      */
	float           *edepah;            /*   "   depth          */
	short           *ieyrah;     /* year         */
	short           *iemoah;     /* month        */
	short           *iedaah;    /* day          */
	short           *iehrah;     /* hour         */
	short           *iemnah;     /* minute       */
	float           *esecah;    /* second       */
	char            ecomah[COMSIZE];      /*      comment line */
	short           *itypah;   /* data type (int,float,...)    */
	long            *ndatah;  /* number of samples            */
	float           *deltah;  /* sampling interval            */
	float           *amaxah; /* maximum amplitude of record  */
	short           *isyrah;     /* year         */
	short           *ismoah;     /* month        */
	short           *isdaah;    /* day          */
	short           *ishrah;     /* hour         */
	short           *ismnah;     /* minute       */
	float           *ssecah;    /* second       */
	float           *rminah;   /* minimum value of abscissa    */
	char            scomah[COMSIZE];      /* comment line      */
	char            logah[LOGSIZE]; /* log of data manipulations */
	float           dumah[NEXTRAS]; /* freebies */

	int iunit;
	float *test;
	float *array;
{
	ahhed head;
	int i,j,ierr=0,ierr1;
	if(iunit > 250) {
		ierr = -2;
		fprintf(stderr,"unit %d is to big!\n",iunit);
		return(ierr);
	}
	if(flag_open[iunit] != 1) {
		ierr = -3;
		fprintf(stderr,"unit %d is not open!\n",iunit);
		return(ierr);
	}
	if(flag_mode[iunit] != 0) {
		ierr = -4;
		fprintf(stderr,"read is not possible on unit %d!\n",iunit);
		return(ierr);
	}
	if (xdr_gethead(&head,&xdr_file[iunit]) == 1) {
		ierr=0;
		head.station.code[5]= '\0';
		head.station.chan[5]= '\0';
		head.station.stype[7]= '\0';
		head.event.ecomment[79]= '\0';
		head.record.rcomment[79]= '\0';
		head.record.log[LOGSIZE-1]= '\0';

		strcpy(codeah,head.station.code);
		strcpy(chanah,head.station.chan);
		strcpy(typeah,head.station.stype);
		*slatah=head.station.slat;
		*slonah=head.station.slon;
		*elevah=head.station.elev;
		*dsah=head.station.DS;
		*a0ah=head.station.A0;

		*xnpah = head.station.cal[0].pole.r;
		*nu1=(int)head.station.cal[0].pole.i;
		*xnzah = head.station.cal[0].zero.r;
		*nu2=(int)head.station.cal[0].zero.i;
		j=1;
		for (i=0;i<2*(NOCALPTS-1);i+=2){
			pz[i]=head.station.cal[j].pole;
			pz[i+1]=head.station.cal[j].zero;
			j++;
		}
		*elatah=head.event.lat;
		*elonah=head.event.lon;
		*edepah=head.event.dep;
		*ieyrah=head.event.ot.yr; 
		*iemoah=head.event.ot.mo;
		*iedaah=head.event.ot.day;
		*iehrah=head.event.ot.hr;
		*iemnah=head.event.ot.mn;
		*esecah=head.event.ot.sec;
		*itypah=head.record.type;
		*ndatah=head.record.ndata;
		*deltah=head.record.delta;
		*amaxah=head.record.maxamp;
		*isyrah=head.record.abstime.yr; 
		*ismoah=head.record.abstime.mo;
		*isdaah=head.record.abstime.day;
		*ishrah=head.record.abstime.hr;
		*ismnah=head.record.abstime.mn;
		*ssecah=head.record.abstime.sec;
		*rminah=head.record.rmin;
	     	strcpy(scomah,head.record.rcomment);
		strcpy(logah,head.record.log);  
		for(i=0;i<NEXTRAS;i++){
			dumah[i]=head.extra[i];
		} 
		ierr=1024;
		if ( head.record.ndata != 0 )
		ierr1 = xdr_getdata(&head,array, &xdr_file[iunit]);
		else
			{
			fprintf(stderr,"load header only\n");
			ierr1=1;
			}
		if(ierr1 <= 0)
		{
			fprintf(stderr,"Error reading data\n");
			ierr = -4;
			return(ierr);
		}
		ierr=ierr+ierr1;
		fprintf(stderr,"\n end of decoding data\n");
	} 
	return(ierr);
}
int xdr_rdahdata(iunit,array,itype,ndata)
char *array;
int ndata,iunit;
short itype;
{
	float *pfloat;
	double	*pdouble;
	complex *pcomplex;
	tensor	*ptensor;
	int i,ierr=0;

	fprintf(stderr," c routine %d %d %d\n",iunit,itype,ndata);

	if(iunit > 250) {
		ierr = -2;
		fprintf(stderr,"unit %d is to big!\n",iunit);
		return(ierr);
	}
	if(flag_open[iunit] != 1) {
		ierr = -3;
		fprintf(stderr,"unit %d is not open!\n",iunit);
		return(ierr);
	}
	if(flag_mode[iunit] != 0) {
		ierr = -4;
		fprintf(stderr,"read is not possible on unit %d!\n",iunit);
		return(ierr);
	}
	switch(itype) {
	case FLOAT:
		pfloat = (float *) array;
		for (i = 0; i < ndata; i++) {
			fprintf(stderr,"-%d",i);
			if (! xdr_float(&xdr_file[iunit], pfloat++) ) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;
	case COMPLEX:
	case VECTOR:
		pcomplex = (complex *) array;
		pcomplex = pcomplex-itype;
		for (i = 0; i < ndata; i++) {
			if (!xdr_float(&xdr_file[iunit], &(pcomplex->r)) ||
			    !xdr_float(&xdr_file[iunit], &(pcomplex++->i))) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;
	case TENSOR:
		ptensor = (tensor *) array;
		ptensor = ptensor-itype;
		for (i = 0; i < ndata; i++) {
			if (!xdr_float(&xdr_file[iunit], &(ptensor->xx)) ||
			    !xdr_float(&xdr_file[iunit], &(ptensor->yy)) ||
			    !xdr_float(&xdr_file[iunit], &(ptensor++->xy))) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;
	case 5:
		pfloat = (float *) array;
		for (i = 0; i < 4 * ndata; i++) {
			if (! xdr_float(&xdr_file[iunit], pfloat++) ) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;
	case DOUBLE:
		pdouble = (double *) array;
		pdouble = pdouble -itype;
		for (i = 0; i < ndata; i++) {
			if (! xdr_double(&xdr_file[iunit], pdouble++) ) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;
	default:
		ierr = -1;
		ah_errno= AE_DTYPE;
		return(ierr);
	}
	fprintf(stderr," fin de xdr_rdahdata:\n");
	return(ierr);
}
int xdr_wrahhead( iunit, 
	codeah,chanah,typeah,slatah,slonah,elevah,dsah,a0ah,xnpah,nu1,xnzah,nu2,  
	pz,elatah,elonah,edepah,ieyrah,iemoah,iedaah,iehrah,iemnah,esecah,ecomah,itypah,ndatah,  
	deltah,amaxah,isyrah,ismoah,isdaah,ishrah,ismnah,ssecah,rminah,scomah,logah,dumah )

	char            codeah[CODESIZE]; /* station code         */
	char            chanah[CHANSIZE]; /* lpz,spn, etc.        */
	char            typeah[STYPESIZE];/* wwssn,hglp,etc.     */
	float           *slatah;           /* station latitude     */
	float           *slonah;           /*    "    longitude    */
	float           *elevah;           /*    "    elevation    */
	float           *dsah;    /* gain */
	float           *a0ah;    /* normalization */
	float 		*xnpah,*xnzah; /* number of pole & zero */
	int 		*nu1,*nu2;     /* filler  */
	complex 	pz[2*(NOCALPTS-1)];      /* calibration info */
	float           *elatah;            /* event latitude       */
	float           *elonah;            /*   "   longitude      */
	float           *edepah;            /*   "   depth          */
	short           *ieyrah;     /* year         */
	short           *iemoah;     /* month        */
	short           *iedaah;    /* day          */
	short           *iehrah;     /* hour         */
	short           *iemnah;     /* minute       */
	float           *esecah;    /* second       */
	char            ecomah[COMSIZE];      /*      comment line */
	short           *itypah;   /* data type (int,float,...)    */
	long            *ndatah;  /* number of samples            */
	float           *deltah;  /* sampling interval            */
	float           *amaxah; /* maximum amplitude of record  */
	short           *isyrah;     /* year         */
	short           *ismoah;     /* month        */
	short           *isdaah;    /* day          */
	short           *ishrah;     /* hour         */
	short           *ismnah;     /* minute       */
	float           *ssecah;    /* second       */
	float           *rminah;   /* minimum value of abscissa    */
	char            scomah[COMSIZE];      /* comment line      */
	char            logah[LOGSIZE]; /* log of data manipulations */
	float           dumah[NEXTRAS]; /* freebies */

	int iunit;
{
	ahhed head;
	int i,j,ierr=0;

	if(iunit > 250) {
		ierr = -2;
		fprintf(stderr,"unit %d is to big!\n",iunit);
		return(ierr);
	}
	if(flag_open[iunit] != 1) {
		ierr = -3;
		fprintf(stderr,"unit %d is not open!\n",iunit);
		return(ierr);
	}
	if(flag_mode[iunit] != 1) {
		ierr = -4;
		fprintf(stderr,"write is not possible on unit %d!\n",iunit);
		return(ierr);
	}
	strcpy(head.station.code,codeah);
	strcpy(head.station.chan,chanah);
	strcpy(head.station.stype,typeah);
	head.station.slat = *slatah;
	head.station.slon = *slonah;
	head.station.elev = *elevah;
	head.station.DS = *dsah;
	head.station.A0 = *a0ah;
	head.station.cal[0].pole.r = *xnpah;
	head.station.cal[0].pole.i=(float)(*nu1);
	head.station.cal[0].zero.r = *xnzah;
	head.station.cal[0].zero.i=(float)(*nu2);
	j=1;
	for (i=0;i<2*(NOCALPTS-1);i+=2){
		head.station.cal[j].pole=pz[i];
		head.station.cal[j].zero=pz[i+1];
		j++;
	}
	head.event.lat = *elatah;
	head.event.lon = *elonah;
	head.event.dep = *edepah;
	head.event.ot.yr = *ieyrah; 
	head.event.ot.mo = *iemoah;
	head.event.ot.day = *iedaah;
	head.event.ot.hr = *iehrah;
	head.event.ot.mn = *iemnah;
	head.event.ot.sec = *esecah;
	head.record.type = *itypah;
	head.record.ndata = *ndatah;
	head.record.delta = *deltah;
	head.record.maxamp = *amaxah;
	head.record.abstime.yr = *isyrah; 
	head.record.abstime.mo = *ismoah;
	head.record.abstime.day = *isdaah;
	head.record.abstime.hr = *ishrah;
	head.record.abstime.mn = *ismnah;
	head.record.abstime.sec = *ssecah;
	head.record.rmin = *rminah;
	strcpy(head.record.rcomment,scomah);
	strcpy(head.record.log,logah);
	for(i=0;i<NEXTRAS;i++){
		head.extra[i]=dumah[i];
	}
	if(xdr_puthead(&head, &xdr_file[iunit]) != 1)
	{
		fprintf(stderr,"Error writing header in %d unit\n",iunit);
		ierr = -1;
		return(ierr);
	}
	ierr=1024;
	return(ierr);
}
int xdr_wrahrecord( iunit, 
	codeah,chanah,typeah,slatah,slonah,elevah,dsah,a0ah,xnpah,nu1,xnzah,nu2,  
	pz,elatah,elonah,edepah,ieyrah,iemoah,iedaah,iehrah,iemnah,esecah,ecomah,itypah,ndatah,  
	deltah,amaxah,isyrah,ismoah,isdaah,ishrah,ismnah,ssecah,rminah,scomah,logah,dumah,array )

	char            codeah[CODESIZE]; /* station code         */
	char            chanah[CHANSIZE]; /* lpz,spn, etc.        */
	char            typeah[STYPESIZE];/* wwssn,hglp,etc.     */
	float           *slatah;           /* station latitude     */
	float           *slonah;           /*    "    longitude    */
	float           *elevah;           /*    "    elevation    */
	float           *dsah;    /* gain */
	float           *a0ah;    /* normalization */
	float 		*xnpah,*xnzah; /* number of pole & zero */
	int 		*nu1,*nu2; /* filler */
	complex 	pz[2*(NOCALPTS-1)];      /* calibration info */
	float           *elatah;            /* event latitude       */
	float           *elonah;            /*   "   longitude      */
	float           *edepah;            /*   "   depth          */
	short           *ieyrah;     /* year         */
	short           *iemoah;     /* month        */
	short           *iedaah;    /* day          */
	short           *iehrah;     /* hour         */
	short           *iemnah;     /* minute       */
	float           *esecah;    /* second       */
	char            ecomah[COMSIZE];      /*      comment line */
	short           *itypah;   /* data type (int,float,...)    */
	long            *ndatah;  /* number of samples            */
	float           *deltah;  /* sampling interval            */
	float           *amaxah; /* maximum amplitude of record  */
	short           *isyrah;     /* year         */
	short           *ismoah;     /* month        */
	short           *isdaah;    /* day          */
	short           *ishrah;     /* hour         */
	short           *ismnah;     /* minute       */
	float           *ssecah;    /* second       */
	float           *rminah;   /* minimum value of abscissa    */
	char            scomah[COMSIZE];      /* comment line      */
	char            logah[LOGSIZE]; /* log of data manipulations */
	float           dumah[NEXTRAS]; /* freebies */

	int iunit;
	char *array;
{
	ahhed head;
	int i,j,ierr=0;

	if(iunit > 250) {
		ierr = -2;
		fprintf(stderr,"unit %d is to big!\n",iunit);
		return(ierr);
	}
	if(flag_open[iunit] != 1) {
		ierr = -3;
		fprintf(stderr,"unit %d is not open!\n",iunit);
		return(ierr);
	}
	if(flag_mode[iunit] != 1) {
		ierr = -4;
		fprintf(stderr,"write is not possible on unit %d!\n",iunit);
		return(ierr);
	}
	strcpy(head.station.code,codeah);
	strcpy(head.station.chan,chanah);
	strcpy(head.station.stype,typeah);
	head.station.slat = *slatah;
	head.station.slon = *slonah;
	head.station.elev = *elevah;
	head.station.DS = *dsah;
	head.station.A0 = *a0ah;
	head.station.cal[0].pole.r = *xnpah;
	head.station.cal[0].pole.i=(float)(*nu1);
	head.station.cal[0].zero.r = *xnzah;
	head.station.cal[0].zero.i=(float)(*nu2);
	j=1;
	for (i=0;i<2*(NOCALPTS-1);i+=2){
		head.station.cal[j].pole=pz[i];
		head.station.cal[j].zero=pz[i+1];
		j++;
	}
	head.event.lat = *elatah;
	head.event.lon = *elonah;
	head.event.dep = *edepah;
	head.event.ot.yr = *ieyrah; 
	head.event.ot.mo = *iemoah;
	head.event.ot.day = *iedaah;
	head.event.ot.hr = *iehrah;
	head.event.ot.mn = *iemnah;
	head.event.ot.sec = *esecah;
	head.record.type = *itypah;
	head.record.ndata = *ndatah;
	head.record.delta = *deltah;
	head.record.maxamp = *amaxah;
	head.record.abstime.yr = *isyrah; 
	head.record.abstime.mo = *ismoah;
	head.record.abstime.day = *isdaah;
	head.record.abstime.hr = *ishrah;
	head.record.abstime.mn = *ismnah;
	head.record.abstime.sec = *ssecah;
	head.record.rmin = *rminah;
	strcpy(head.record.rcomment,scomah);
	strcpy(head.record.log,logah);
	for(i=0;i<NEXTRAS;i++){
		head.extra[i]=dumah[i];
	}
	if(xdr_puthead(&head, &xdr_file[iunit]) != 1)
	{
		fprintf(stderr,"Error writing header in %d unit\n",iunit);
		ierr = -1;
		return(ierr);
	}
	ierr=1024;
	if(xdr_putdata(&head,(char *)array, &xdr_file[iunit]) < 0)
	{
		fprintf(stderr,"Error writing data in %d unit\n",iunit);
		ierr = -2;
		return(ierr);
	}
	ierr=ierr + *ndatah;
	return(ierr);
}
int xdr_wrahdata(iunit,array,itype,ndata)
char *array;
short itype;
int ndata,iunit;
{
	float *pfloat;
	double	*pdouble;
	complex *pcomplex;
	tensor	*ptensor;
	int i,ierr = 0;

	if(iunit > 250) {
		fprintf(stderr,"unit %d is to big!\n",iunit);
		ierr = -2;
		return(ierr);
	}
	if(flag_open[iunit] != 1) {
		fprintf(stderr,"unit %d is not open!\n",iunit);
		ierr = -3;
		return(ierr);
	}
	if(flag_mode[iunit] != 1) {
		fprintf(stderr,"write is not possible on unit %d!\n",iunit);
		ierr = -4;
		return(ierr);
	}
	switch(itype) {
	case FLOAT:
		pfloat = (float *) array;
		for (i = 0; i < ndata; i++) {
			if (! xdr_float(&xdr_file[iunit], pfloat++) ) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;
	case COMPLEX:
	case VECTOR:
		pcomplex = (complex *) array;
		for (i = 0; i < ndata; i++) {
			if (!xdr_float(&xdr_file[iunit], &(pcomplex->r)) ||
			    !xdr_float(&xdr_file[iunit], &(pcomplex++->i))) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;
	case TENSOR:
		ptensor = (tensor *) array;
		for (i = 0; i < ndata; i++) {
			if (!xdr_float(&xdr_file[iunit], &(ptensor->xx)) ||
			    !xdr_float(&xdr_file[iunit], &(ptensor->yy)) ||
			    !xdr_float(&xdr_file[iunit], &(ptensor++->xy))) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;
	case 5:
		pfloat = (float *) array;
		for (i = 0; i < 4 * ndata; i++) {
			if (! xdr_float(&xdr_file[iunit], pfloat++) ) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;
	case DOUBLE:
		pdouble = (double *) array;
		for (i = 0; i < ndata; i++) {
			if (! xdr_double(&xdr_file[iunit], pdouble++) ) {
				ah_errno= AE_RDATA;
				ierr = -1;
				return(ierr);
			}
			++ierr;
		}
		break;

	default:
		ierr = -1;
		ah_errno= AE_DTYPE;
		return(ierr);
	}
	return(ierr);
}



/*	mkdatspace
 *		allocates enough space for the data array, and
 *		returns a pointer to the memory location, or
 *		NULL if failure.
 *	returns:
 *			character pointer	->	success
 *			NULL			->	failure
 */
char	*mkdatspace(head)
	ahhed	*head;
{
	char	*calloc();
	
	return(calloc((unsigned)head->record.ndata,(unsigned)size(head)));
}



get_null_head(hed)
ahhed	*hed;
{
int	i;

	strcpy(hed->station.code,"null");
	strcpy(hed->station.chan,"null");
	strcpy(hed->station.stype,"null");
	hed->station.slat= 0.0;
	hed->station.slon= 0.0;
	hed->station.elev= 0.0;
	hed->station.DS= 0.0;
	hed->station.A0= 0.0;
	for(i=0; i< NOCALPTS; ++i)
	{
		hed->station.cal[i].pole.r= 0.0;
		hed->station.cal[i].pole.i= 0.0;
		hed->station.cal[i].zero.r= 0.0;
		hed->station.cal[i].zero.i= 0.0;
	}

	hed->event.lat= 0.0;
	hed->event.lon= 0.0;
	hed->event.dep= 0.0;
	hed->event.ot.yr= (short)0;
	hed->event.ot.mo= (short)0;
	hed->event.ot.day= (short)0;
	hed->event.ot.hr= (short)0;
	hed->event.ot.mn= (short)0;
	hed->event.ot.sec= 0.0;
	strcpy(hed->event.ecomment,"null");

	hed->record.type= (short)0;
	hed->record.ndata= 0L;
	hed->record.delta= 0.0;
	hed->record.maxamp= 0.0;
	hed->record.abstime.yr= (short)0;
	hed->record.abstime.mo= (short)0;
	hed->record.abstime.day= (short)0;
	hed->record.abstime.hr= (short)0;
	hed->record.abstime.mn= (short)0;
	hed->record.abstime.sec= 0.0;
	hed->record.rmin= 0.0;
	strcpy(hed->record.rcomment,"null");
	strcpy(hed->record.log,"null");

	for(i=0; i< NEXTRAS; ++i)
		hed->extra[i]= 0.0;

	return;
}


/*	size
 *		returns the size (in bytes) of the data type given by
 *		head->record.type.
 *	returns:
 *			size of data type	->	no error
 *			-1			->	unknown data type
 */
int	size(head)
	ahhed	*head;
{
	int	type_size = 0;

	switch(head->record.type)
	{
	case 1:		/* real time series */
		type_size= sizeof(float);
		break;
	case 2:		/* complex time series */
		type_size= sizeof(complex);
		break;
	case 3:		/* real x,y pairs */
		type_size= sizeof(vector);
		break;
	case 4:		/* x real, y complex, or real x,y,z */
		type_size= sizeof(tensor);
		break;
	case 5:		/* complex x,y pairs */
		type_size= 2*sizeof(complex);
		break;
	case 6:		/* double */
		type_size=sizeof(double);
		break;
	default:	/* unknown data type */
		type_size= -1;
		ah_errno= AE_DTYPE;
		break;
	}
	return(type_size);
}

/*	gethead
 *		gets the next header from the stream pointed to by
 *		file_pt and returns this header in the structure head.
 *		file_pt is assumed to be positioned at the next header,
 *		and does not search.
 *	returns:
 *			1		->	no error
 *			-1		->	not enough head to read
 *			-2		->	bad data type
 */
int	gethead(head,file_pt)
	FILE	*file_pt;
	ahhed	*head;
{
	int	ierr = 0;

	if((ierr = fread((char *)head,sizeof(ahhed),1,file_pt)) == 1)
	{
		if((head->record.type < TYPEMIN) || (head->record.type > TYPEMAX))
		{
			get_null_head(head);
			ierr = -2;	/* bad data type */
			ah_errno= AE_DTYPE;
		}
	}
	else		/* not enough head */
	{
		get_null_head(head);
		ierr = -1;
		ah_errno= AE_RHED;
	}
	return(ierr);
}



/*	puthead
 *		writes the header head onto the stream pointed to by
 *		file_pt.
 *	returns:
 *			1		->	no error
 *			-1		->	error writing header
 */
int	puthead(head,file_pt)
	FILE	*file_pt;
	ahhed	*head;
{
	int	ierr = 0;

	if((ierr= fwrite((char *)head,sizeof(ahhed),1,file_pt)) != 1)
	{
		ah_errno= AE_WHED;
		ierr= -1;
	}
	return(ierr);
}

/*	tohead
 *		positions the read/write head to the beginning of the
 *		n-th header in the file pointed to by file_pt.
 *	returns:
 *			n	->	no error
 *			-1	->	not enough heads
 *			-2	->	bad seek
 */
int	tohead(n,file_pt)
	FILE	*file_pt;
	int	n;
{
	ahhed	head;
	int	i,ierr;

	rewind(file_pt);
	for(i=1; i<n; ++i)
	{
		if(gethead(&head,file_pt) == 1)
		{
			if(fseek(file_pt,(long)(head.record.ndata)*(size(&head)),1) == -1)
			{
				ierr = -2;	/* bad seek */
				ah_errno= AE_RHED;
				return(ierr);
			}
		}
		else
		{
			ierr = -1;	/* not enough head */
			ah_errno= AE_RHED;
			return(ierr);
		}
	}
	return(i);	/* success */
}



/*	getdata
 *		reads from the file pointed to by file_pt into
 *		the array pointed to by array.  It assumes that
 *		the read/write head is positioned correctly 
 *		(i.e., right after the header), and does not
 *		search.  Works for any allowed data type.
 *	returns:
 *			number of elements read	->	OK
 *			-1			->	error
 */
int	getdata(head,array,file_pt)
	ahhed	*head;
	char	*array;
	FILE	*file_pt;

{
	int ierr = 0;

	if((ierr = fread(array,size(head),(int)head->record.ndata,file_pt)) != (int)head->record.ndata)
	{
		ah_errno= AE_RDATA;
		ierr = -1;
	}

	return(ierr);
}


/*	putdata
 *		writes array to the file pointed to by
 *		file_pt.  Works for any allowed data type.
 *	returns:
 *			number of elements written	->	OK
 *			-1			->	error
 */
int	putdata(head,array,file_pt)
	ahhed	*head;
	char	*array;
	FILE	*file_pt;
{
	int	ierr = 0;

	if((ierr = fwrite(array,size(head),(int)head->record.ndata,file_pt)) != (int)head->record.ndata)
	{
		ah_errno= AE_WDATA;
		ierr = -1;
	}
	return(ierr);
}


/*	putrecord
 *		writes head and array to the file pointed to by
 *		file_pt.  Works for any allowed data type.
 *	returns:
 *			0	->	OK
 *			-1	->	error writing header
 *			-2	->	error writing data
 */
int	putrecord(head,array,file_pt)
	ahhed	*head;
	char	*array;
	FILE	*file_pt;
{
	int	ierr = 0;

	(puthead(head,file_pt) == 1) ? ((putdata(head,array,file_pt) < 0) ? (ierr = -2) : (ierr = 0)) : (ierr = -1);
	if(ierr)
		ah_errno= AE_WRECORD;

	return(ierr);
}


/*	getrecord
 *		gets header and data from the file pointed to by
 *		file_pt and puts them in head and array.  It assumes
 *		that the read/write head is positioned at the beginning
 *		of the header, and does not search.  Obviously, calling
 *		routine must have allocated enough space.
 *	returns:
 *			0	->	OK
 *			-1	->	error reading header
 *			-2	->	error reading data
 */
int	getrecord(head,array,file_pt)
	ahhed	*head;
	char	*array;
	FILE	*file_pt;
{
	int	ierr = 0;

	(gethead(head,file_pt) == 1) ? ((getdata(head,array,file_pt) < 0) ? (ierr = -2) : (ierr = 0)) : (ierr = -1);
	if(ierr)
		ah_errno= AE_RRECORD;
	return(ierr);
}

/*
 *	getrecord2
 *		gets header and data from the file pointed to by
 *		file_pt and puts them in head and array.  It assumes
 *		that the read/write head is positioned at the beginning
 *		of the header, and does not search (although it does
 *		some error checking).  Space for array is allocated, so
 *		be sure to pass a pointer to the data pointer. Got it?
 *	returns:
 *			0	->	ok
 *			-1	->	error reading record
 *			-2	->	error allocating space for data
 */
int	getrecord2(head,array,file_pt)
	ahhed	*head;
	char	**array;
	FILE	*file_pt;
{
	int	ierr = 0;
	int	gethead();
	char	*mkdatspace();

	if(gethead(head, file_pt) != 1) {
		ierr = -1;
		return(ierr);
	}

	*array= mkdatspace(head);
	if(*array == NULL) {
		ierr= -2;
		return(ierr);
	}

	if(getdata(head,*array,file_pt) < 0)
		ierr= -1;

	return(ierr);
}


/*	gogethead
 *		gets n-th header from the stream pointed to by
 *		file_pt and returns this header in the structure
 *		head.
 *	returns:
 *			0	->	OK
 *			-1	->	stream not long enough
 *			-2	->	error reading header
 */
int	gogethead(n,head,file_pt)
	int	n;
	ahhed	*head;
	FILE	*file_pt;
{
	int	ierr = 0;

	(tohead(n,file_pt) == n) ? ((gethead(head,file_pt) < 1) ? (ierr = -2) : (ierr = 0)) : (ierr = -1);
	return(ierr);
}


/*	gogetrecord
 *		gets n-th record (header and data) from the stream
 *		pointed to by file_pt and places it in head and array.
 *		Calling routine must allocate enough space.
 *	returns:
 *			0	->	OK
 *			-1	->	stream not long enough
 *			-2	->	error reading record
 */
int	gogetrecord(n,head,array,file_pt)
	int	n;
	ahhed	*head;
	char	*array;
	FILE	*file_pt;

{
	int	ierr = 0;

	(tohead(n,file_pt) == n) ? ((getrecord(head,array,file_pt) < 0) ? (ierr = -2) : (ierr = 0)) : (ierr = -1);
	return(ierr);
}

/* logger adds a 10 character comment to the log section of the header
 * comment should be passed as a character pointer must be terminated
 * by a ';' and a `/0`
 * returns:
 *	logger =  0  -> log info added to header structure
 *	logger = -1  -> no ';', added
 *	logger = -2  -> input string greater than LOGENT
 *                      input truncated to allowable limit
 *	logger = -3  -> attempt to make log string greater than LOGSIZE
 *                      input comment truncated to fit 
 *
 *			written by Tom Boyd   6/10/85
 */

int logger(char_pt,head_pt)
ahhed *head_pt;
char *char_pt;

{
	int org,in,err,diff;
/*	char *strncat();*/

	err=0;

/* find length of log array and input array  */

	org=strlen(head_pt->record.log);  /*log array*/
	in=strlen(char_pt);  /*input array*/

/* check for a terminating ':' in the input array */

	if(*(char_pt+in-1) != ';')
	{                   /* no semicolon----add it*/
		err=(-1);
		*(char_pt+in)=';';
		*(char_pt+in+1)='\0';
		in+=1;
	}

/* check the length of the input array */

	if(in > LOGENT)
	{                  /* entry length too long-----truncate it*/
		err=(-2);
		*(char_pt+LOGENT-1)=';';
		*(char_pt+LOGENT)='\0';
		in=LOGENT;
	}

/* check combined length of array and new input and add it */

	diff=LOGSIZE-(org+in);
	if(diff == -in) return(-3);  /* no room left in log array */
	if(diff < 0)diff*=(-1),err=(-3);  /*partial room left----use it */	
	strncat(head_pt->record.log,char_pt,diff); /* cat two strings */

	return(err);
}



/*	out_is_tty
 *		determines whether stdout is being sent to screen.
 *	returns:
 *			0	->	stdout is not tty
 *			1	->	stdout is tty
 */
int	out_is_tty()

{

	if(isatty(1))	/* sun specific --- stdout */
	{
		ah_errno= AE_TTYOUT;
		return(1);
	}
	return(0);
}


/*	in_is_tty
 *		determines whether stdin is tty
 *	returns:
 *			0	->	stdin is not tty
 *			1	->	stdin is tty
 */
int	in_is_tty()
{

	if(isatty(0))	/* sun specific --- stdin */
	{
		ah_errno= AE_TTYIN;
		return(1);
	}
	return(0);
}

/* acpy(from,to,nbytes) copies nbytes from the array "from" to the
 *	array "to".
 */
acpy(from,to,nbytes)
char	*from;
char	*to;
unsigned	nbytes;
{
	while(nbytes--)
		*from++ = *to++;
	return;
}


#define	MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define	MIN(a,b)	(((a) < (b)) ? (a) : (b))

/*
 *	maxamp
 *		determines the maximum absolute amplitude of the data array, and
 *		places that number in head.record.maxamp.
 *	returns:
 *			0	->	ok
 *			-1	->	error
 */
int	maxamp(head,data)
 	ahhed	*head;
 	char	*data;
{
	float	*fpt;
	double *dpt, dmin, dmax;
	float	max,min;
	long	n_data_pts;

	switch(head->record.type)
	{
	case FLOAT:
		n_data_pts= head->record.ndata;
		break;
	case COMPLEX:
	case VECTOR:
		n_data_pts= 2 * head->record.ndata;
		break;
	case TENSOR:
		n_data_pts= 3 * head->record.ndata;
		break;
	case 5:
		n_data_pts= 4 * head->record.ndata;
		break;
	case DOUBLE:
		n_data_pts= head->record.ndata;
		break;
	default:
		ah_errno= AE_DTYPE;
		return(-1);
		break;
	}

	if (head->record.type == DOUBLE) {
		dpt= (double *)data;
		dmax= dmin= *dpt;
		while(n_data_pts--)
		{
			dmax= MAX(dmax,*dpt);
			dmin= MIN(dmin,*dpt);
			++dpt;
		}
		((fabs(dmax) > fabs(dmin)) ? (head->record.maxamp= (float) dmax) : (head->record.maxamp= (float) -dmin));
	}
	else {
		fpt= (float *)data;
		max= min= *fpt;
		while(n_data_pts--)
		{
			max= MAX(max,*fpt);
			min= MIN(min,*fpt);
			++fpt;
		}
		((fabs((double)max) > fabs((double)min)) ? (head->record.maxamp= max) : (head->record.maxamp= -min));
	}

	return(0);
}

